@@ -17,16 +17,16 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#ifndef _tdsversion_h_
-#define _tdsversion_h_
+#ifndef _tdsguard_fWmaUBUxPCWj6BPyET5377_
+#define _tdsguard_fWmaUBUxPCWj6BPyET5377_
 
-/* $Id: 9640daf619734147e3b5ff6a1dad56ad305ffa51 $ */
+/* $Id: c69b1270e9bbde50d3708cbfbf8921f8e849c928 $ */
 
-#define TDS_VERSION_NO "freetds v1.4.27"
+#define TDS_VERSION_NO "freetds v1.5"
 
 #define TDS_VERSION_MAJOR 1
-#define TDS_VERSION_MINOR 4
-#define TDS_VERSION_SUBVERSION 27
-#define TDS_VERSION_BUILD_NUMBER 838
+#define TDS_VERSION_MINOR 5
+#define TDS_VERSION_SUBVERSION 0
+#define TDS_VERSION_BUILD_NUMBER 107
 
 #endif
