@@ -39,6 +39,7 @@
 
 #include <freetds/time.h>
 #include <freetds/sysdep_private.h>
+#include <freetds/utils/path.h>
 
 #if HAVE_LOCALE_H
 #include <locale.h>
@@ -48,6 +49,7 @@
 #include <sybdb.h>
 
 #include <freetds/replacements.h>
+#include <freetds/macros.h>
 
 typedef struct
 {
@@ -87,7 +89,7 @@ static RETCODE set_textsize(DBPROCESS *dbproc, int textsize);
 static int err_handler(DBPROCESS *, int, int, int, char *, char *);
 static int msg_handler(DBPROCESS *, DBINT, int, int, char *, char *, char *, int);
 
-int tdsdump_open(const char *filename);
+int tdsdump_open(const tds_dir_char *filename);
 
 int
 main(int argc, char **argv)
@@ -237,7 +239,7 @@ process_parameters(int argc, char **argv, BCPPARAMDATA * pdata)
 			pdata->owner = strdup(optarg);
 			break;
 		case 'd':
-			tdsdump_open("stderr");
+			tdsdump_open(TDS_DIR("stderr"));
 			break;
 		case 'S':
 			pdata->Sflag++;
@@ -564,7 +566,7 @@ check_table_structures(char *sobjname, char *dobjname, DBPROCESS * dbsrc, DBPROC
 			continue;
 		}
 
-		if (src_coltype != dest_coltype || src_collen != dest_collen) {
+		if (src_coltype != dest_coltype || src_collen > dest_collen) {
 			fprintf(stderr, "COLUMN TYPE MISMATCH: column %d\n", i);
 			fprintf(stderr, "source: type %d, length %d\n", src_coltype, src_collen);
 			fprintf(stderr, "dest  : type %d, length %d\n", dest_coltype, dest_collen);
@@ -817,7 +819,8 @@ pusage(void)
 }
 
 static int
-err_handler(DBPROCESS * dbproc, int severity, int dberr, int oserr, char *dberrstr, char *oserrstr)
+err_handler(DBPROCESS * dbproc TDS_UNUSED, int severity, int dberr, int oserr TDS_UNUSED,
+	    char *dberrstr, char *oserrstr TDS_UNUSED)
 {
 	if (dberr) {
 		fprintf(stderr, "Msg %d, Level %d\n", dberr, severity);
@@ -831,7 +834,8 @@ err_handler(DBPROCESS * dbproc, int severity, int dberr, int oserr, char *dberrs
 }
 
 static int
-msg_handler(DBPROCESS * dbproc, DBINT msgno, int msgstate, int severity, char *msgtext, char *srvname, char *procname, int line)
+msg_handler(DBPROCESS * dbproc TDS_UNUSED, DBINT msgno, int msgstate, int severity,
+	    char *msgtext, char *srvname, char *procname, int line)
 {
 	/*
 	 * If it's a database change message, we'll ignore it.
