@@ -48,6 +48,7 @@
 #include <freetds/convert.h>
 #include <freetds/bytes.h>
 #include <freetds/replacements.h>
+#include <freetds/utils.h>
 
 typedef unsigned short utf16_t;
 
@@ -63,12 +64,6 @@ struct tds_time
 };
 
 static TDS_INT tds_convert_int(TDS_INT num, int desttype, CONV_RESULT * cr);
-static TDS_INT tds_convert_int1(const TDS_TINYINT * src, int desttype, CONV_RESULT * cr);
-static TDS_INT tds_convert_int2(const TDS_SMALLINT * src, int desttype, CONV_RESULT * cr);
-static TDS_INT tds_convert_uint2(const TDS_USMALLINT * src, int desttype, CONV_RESULT * cr);
-static TDS_INT tds_convert_int4(const TDS_INT* src, int desttype, CONV_RESULT * cr);
-static TDS_INT tds_convert_uint4(const TDS_UINT * src, int desttype, CONV_RESULT * cr);
-static TDS_INT tds_convert_int8(const TDS_INT8 * src, int desttype, CONV_RESULT * cr);
 static TDS_INT tds_convert_uint8(const TDS_UINT8 * src, int desttype, CONV_RESULT * cr);
 static int string_to_datetime(const char *datestr, TDS_UINT len, int desttype, CONV_RESULT * cr);
 static bool is_dd_mon_yyyy(char *t);
@@ -86,6 +81,7 @@ static const char *parse_numeric(const char *buf, const char *pend,
 #define IS_TINYINT(x) ( -1 < (x) && (x) < 0x100 )
 #define IS_SMALLINT(x) ( -32769 < (x) && (x) < 32768 )
 #define IS_USMALLINT(x) ( -1 < (x) && (x) < 65536 )
+#define IS_SINT1(x) ( -129 < (x) && (x) < 128 )
 
 /*
  * Macros for integer number checks.
@@ -99,8 +95,8 @@ static const char *parse_numeric(const char *buf, const char *pend,
 #define INT_IS_INT(x) (TDS_INT_MIN <= (x) && (x) <= TDS_INT_MAX)
 #define TDS_UINT_MAX 4294967295u
 #define IS_UINT(x) (-1 < (TDS_INT8)(x) && (x) < (TDS_INT8) TDS_UINT_MAX + 1)
-#define TDS_INT8_MIN (-(((TDS_INT8)1)<<62) -(((TDS_INT8)1)<<62))
-#define TDS_INT8_MAX ((((TDS_INT8) 0x7fffffff) << 32) + (TDS_INT8) 0xffffffffu)
+#define TDS_INT8_MIN (-INT64_C(9223372036854775807)-1)
+#define TDS_INT8_MAX INT64_C(9223372036854775807)
 
 /*
  * Macros for floating point number checks.
@@ -267,6 +263,7 @@ tds_convert_binary(const TDS_UCHAR * src, TDS_INT srclen, int desttype, CONV_RES
 		return (srclen * 2);
 		break;
 	case SYBINT1:
+	case SYBSINT1:
 	case SYBUINT1:
 	case SYBINT2:
 	case SYBUINT2:
@@ -373,6 +370,14 @@ tds_convert_char(const TDS_CHAR * src, TDS_UINT srclen, int desttype, CONV_RESUL
 		return srclen;
 		break;
 
+	case SYBSINT1:
+		if ((rc = string_to_int(src, src + srclen, &tds_i)) < 0)
+			return rc;
+		if (!IS_SINT1(tds_i))
+			return TDS_CONVERT_OVERFLOW;
+		cr->ti = tds_i;
+		return sizeof(TDS_TINYINT);
+		break;
 	case SYBINT1:
 	case SYBUINT1:
 		if ((rc = string_to_int(src, src + srclen, &tds_i)) < 0)
@@ -456,8 +461,8 @@ tds_convert_char(const TDS_CHAR * src, TDS_UINT srclen, int desttype, CONV_RESUL
 			mynumber[i++] = '0';
 
 		/* convert number and check for overflow */
-		if (string_to_int8(mynumber, mynumber + i, &mymoney) < 0)
-			return TDS_CONVERT_OVERFLOW;
+		if ((rc = string_to_int8(mynumber, mynumber + i, &mymoney)) < 0)
+			return rc;
 
 		if (desttype == SYBMONEY) {
 			cr->m.mny = mymoney;
@@ -571,7 +576,13 @@ tds_convert_bit(const TDS_CHAR * src, int desttype, CONV_RESULT * cr)
 }
 
 static TDS_INT
-tds_convert_int1(const TDS_TINYINT * src, int desttype, CONV_RESULT * cr)
+tds_convert_int1(const int8_t * src, int desttype, CONV_RESULT * cr)
+{
+	return tds_convert_int(*src, desttype, cr);
+}
+
+static TDS_INT
+tds_convert_uint1(const TDS_TINYINT * src, int desttype, CONV_RESULT * cr)
 {
 	return tds_convert_int(*src, desttype, cr);
 }
@@ -641,6 +652,12 @@ tds_convert_int(TDS_INT num, int desttype, CONV_RESULT * cr)
 		sprintf(tmp_str, "%d", num);
 		return string_to_result(desttype, tmp_str, cr);
 		break;
+	case SYBSINT1:
+		if (!IS_SINT1(num))
+			return TDS_CONVERT_OVERFLOW;
+		cr->ti = (TDS_TINYINT) num;
+		return sizeof(TDS_TINYINT);
+		break;
 	case SYBINT1:
 	case SYBUINT1:
 		if (!IS_TINYINT(num))
@@ -740,6 +757,7 @@ tds_convert_int8(const TDS_INT8 *src, int desttype, CONV_RESULT * cr)
 		return string_to_result(desttype, tmp_str, cr);
 		break;
 	case SYBINT1:
+	case SYBSINT1:
 	case SYBUINT1:
 	case SYBINT2:
 	case SYBUINT2:
@@ -818,6 +836,7 @@ tds_convert_uint8(const TDS_UINT8 *src, int desttype, CONV_RESULT * cr)
 		return string_to_result(desttype, tmp_str, cr);
 		break;
 	case SYBINT1:
+	case SYBSINT1:
 	case SYBUINT1:
 	case SYBINT2:
 	case SYBUINT2:
@@ -890,6 +909,21 @@ tds_convert_numeric(const TDS_NUMERIC * src, int desttype, CONV_RESULT * cr)
 			return TDS_CONVERT_FAIL;
 		return string_to_result(desttype, tmpstr, cr);
 		break;
+	case SYBSINT1:
+		cr->n = *src;
+		ret = tds_numeric_change_prec_scale(&(cr->n), 3, 0);
+		if (ret < 0)
+			return ret;
+		if (cr->n.array[1])
+			return TDS_CONVERT_OVERFLOW;
+		i = cr->n.array[2];
+		if (cr->n.array[0])
+			i = -i;
+		if (i != 0 && ((i >> 7) ^ cr->n.array[0]) & 1)
+			return TDS_CONVERT_OVERFLOW;
+		cr->ti = (TDS_TINYINT) i;
+		return sizeof(TDS_TINYINT);
+		break;
 	case SYBINT1:
 	case SYBUINT1:
 		cr->n = *src;
@@ -1042,7 +1076,6 @@ tds_convert_numeric(const TDS_NUMERIC * src, int desttype, CONV_RESULT * cr)
 		cr->r = (TDS_REAL) atof(tmpstr);
 		return 4;
 		break;
-		/* TODO conversions to money */
 		/* conversions not allowed */
 	case SYBUNIQUE:
 	case SYBDATETIME4:
@@ -1059,13 +1092,13 @@ tds_convert_money4(const TDSCONTEXT * tds_ctx, const TDS_MONEY4 * src, int destt
 {
 	TDS_MONEY4 mny;
 	long dollars;
-	char tmp_str[33];
-	char *p;
 
 	mny = *src;
 	switch (desttype) {
 	case TDS_CONVERT_CHAR:
 	case CASE_ALL_CHAR: {
+		char tmp_str[33];
+		char *p;
 		unsigned dollars;
 		/*
 		 * round to 2 decimal digits
@@ -1091,6 +1124,13 @@ tds_convert_money4(const TDSCONTEXT * tds_ctx, const TDS_MONEY4 * src, int destt
 		}
 		return string_to_result(desttype, tmp_str, cr);
 		} break;
+	case SYBSINT1:
+		dollars = mny.mny4 / 10000;
+		if (!IS_SINT1(dollars))
+			return TDS_CONVERT_OVERFLOW;
+		cr->ti = (TDS_TINYINT) dollars;
+		return sizeof(TDS_TINYINT);
+		break;
 	case SYBINT1:
 	case SYBUINT1:
 		dollars = mny.mny4 / 10000;
@@ -1190,6 +1230,12 @@ tds_convert_money(const TDSCONTEXT * tds_ctx, const TDS_MONEY * src, int desttyp
 		s = tds_money_to_string((const TDS_MONEY *) src, tmpstr, tds_ctx->money_use_2_digits);
 		return string_to_result(desttype, s, cr);
 		break;
+	case SYBSINT1:
+		if (mymoney <= -129 * 10000 || mymoney >= 128 * 10000)
+			return TDS_CONVERT_OVERFLOW;
+		cr->ti = (int8_t) (((TDS_INT) mymoney) / 10000);
+		return sizeof(TDS_TINYINT);
+		break;
 	case SYBINT1:
 	case SYBUINT1:
 		if (mymoney <= -10000 || mymoney >= 256 * 10000)
@@ -1322,13 +1368,14 @@ tds_convert_datetimeall(const TDSCONTEXT * tds_ctx, int srctype, const TDS_DATET
 		return sizeof(TDS_UINT8);
 	case SYB5BIGDATETIME:
 		cr->bigtime = dta->time / 10u
-			      + (dta->date + BIGDATETIME_BIAS) * ((TDS_UINT8) 86400u * 1000000u);
+			      + (dta->date + BIGDATETIME_BIAS) * (UINT64_C(86400) * 1000000u);
 		return sizeof(TDS_UINT8);
 		/* conversions not allowed */
 	case SYBUNIQUE:
 	case SYBBIT:
 	case SYBBITN:
 	case SYBINT1:
+	case SYBSINT1:
 	case SYBUINT1:
 	case SYBINT2:
 	case SYBUINT2:
@@ -1385,7 +1432,7 @@ tds_convert_datetime(const TDSCONTEXT * tds_ctx, const TDS_DATETIME * dt, int de
 		if (desttype != SYBMSDATE) {
 			cr->dta.has_time = 1;
 			cr->dta.time_prec = 3;
-			cr->dta.time = (((TDS_UINT8) dt->dttime) * 20 + 3) / 6 * 10000u;
+			cr->dta.time = (dt->dttime * UINT64_C(20) + 3) / 6 * 10000u;
 		}
 		if (desttype != SYBMSTIME) {
 			cr->dta.has_date = 1;
@@ -1393,17 +1440,18 @@ tds_convert_datetime(const TDSCONTEXT * tds_ctx, const TDS_DATETIME * dt, int de
 		}
 		return sizeof(TDS_DATETIMEALL);
 	case SYB5BIGTIME:
-		cr->bigtime = (((TDS_UINT8) dt->dttime) * 20u + 3u) / 6u * 1000u;
+		cr->bigtime = (dt->dttime * UINT64_C(20) + 3u) / 6u * 1000u;
 		return sizeof(TDS_BIGTIME);
 	case SYB5BIGDATETIME:
-		cr->bigdatetime = (((TDS_UINT8) dt->dttime) * 20u + 3u) / 6u * 1000u
-				  + (dt->dtdays + BIGDATETIME_BIAS) * ((TDS_UINT8) 86400u * 1000000u);
+		cr->bigdatetime = (dt->dttime * UINT64_C(20) + 3u) / 6u * 1000u
+				  + (dt->dtdays + BIGDATETIME_BIAS) * (UINT64_C(86400) * 1000000u);
 		return sizeof(TDS_BIGDATETIME);
 		/* conversions not allowed */
 	case SYBUNIQUE:
 	case SYBBIT:
 	case SYBBITN:
 	case SYBINT1:
+	case SYBSINT1:
 	case SYBUINT1:
 	case SYBINT2:
 	case SYBUINT2:
@@ -1451,8 +1499,8 @@ tds_convert_time(const TDSCONTEXT * tds_ctx, const TDS_TIME * time, int desttype
 	memset(&dta, 0, sizeof(dta));
 	dta.has_time = 1;
 	dta.time_prec = 3;
-	dta.time = ((TDS_UINT8) *time * 100000 + 2) / 3;
-	dta.time = (((TDS_UINT8) *time) * 20 + 3) / 6 * 10000;
+	dta.time = (*time * UINT64_C(100000) + 2) / 3;
+	dta.time = (*time * UINT64_C(20) + 3) / 6 * 10000;
 	return tds_convert_datetimeall(tds_ctx, SYBMSTIME, &dta, desttype, cr);
 }
 
@@ -1487,7 +1535,7 @@ tds_convert_bigtime(const TDSCONTEXT * tds_ctx, const TDS_BIGTIME * bigtime, int
 	memset(&dta, 0, sizeof(dta));
 	dta.time_prec = 6;
 	dta.has_time = 1;
-	dta.time = *bigtime % ((TDS_UINT8) 86400u * 1000000u) * 10u;
+	dta.time = *bigtime % (UINT64_C(86400) * 1000000u) * 10u;
 	return tds_convert_datetimeall(tds_ctx, SYBMSTIME, &dta, desttype, cr);
 }
 
@@ -1507,8 +1555,8 @@ tds_convert_bigdatetime(const TDSCONTEXT * tds_ctx, const TDS_BIGDATETIME * bigd
 	memset(&dta, 0, sizeof(dta));
 	dta.time_prec = 6;
 	dta.has_time = 1;
-	dta.time = bdt % ((TDS_UINT8) 86400u * 1000000u) * 10u;
-	bdt /= (TDS_UINT8) 86400u * 1000000u;
+	dta.time = bdt % (UINT64_C(86400) * 1000000u) * 10u;
+	bdt /= UINT64_C(86400) * 1000000u;
 	dta.has_date = 1;
 	dta.date = bdt - BIGDATETIME_BIAS;
 	return tds_convert_datetimeall(tds_ctx, SYBMSDATETIME2, &dta, desttype, cr);
@@ -1533,6 +1581,12 @@ tds_convert_real(const TDS_REAL* src, int desttype, CONV_RESULT * cr)
 		sprintf(tmp_str, "%.9g", the_value);
 		return string_to_result(desttype, tmp_str, cr);
 		break;
+	case SYBSINT1:
+		if (!IS_SINT1(the_value))
+			return TDS_CONVERT_OVERFLOW;
+		cr->ti = (TDS_TINYINT) the_value;
+		return sizeof(TDS_TINYINT);
+		break;
 	case SYBINT1:
 	case SYBUINT1:
 		if (!IS_TINYINT(the_value))
@@ -1642,6 +1696,12 @@ tds_convert_flt8(const TDS_FLOAT* src, int desttype, CONV_RESULT * cr)
 		sprintf(tmp_str, "%.17g", the_value);
 		return string_to_result(desttype, tmp_str, cr);
 		break;
+	case SYBSINT1:
+		if (!IS_SINT1(the_value))
+			return TDS_CONVERT_OVERFLOW;
+		cr->ti = (TDS_TINYINT) the_value;
+		return sizeof(TDS_TINYINT);
+		break;
 	case SYBINT1:
 	case SYBUINT1:
 		if (!IS_TINYINT(the_value))
@@ -1759,6 +1819,7 @@ tds_convert_unique(const TDS_CHAR * src, int desttype, CONV_RESULT * cr)
 	case SYBBIT:
 	case SYBBITN:
 	case SYBINT1:
+	case SYBSINT1:
 	case SYBUINT1:
 	case SYBINT2:
 	case SYBUINT2:
@@ -1792,6 +1853,7 @@ tds_convert_to_binary(int srctype, const TDS_CHAR * src, TDS_UINT srclen, int de
 	case SYBBIT:
 	case SYBBITN:
 	case SYBINT1:
+	case SYBSINT1:
 	case SYBUINT1:
 		len = 1;
 		break;
@@ -1940,9 +2002,12 @@ tds_convert(const TDSCONTEXT *tds_ctx, int srctype, const void *src, TDS_UINT sr
 	case SYBBITN:
 		length = tds_convert_bit(src, desttype, cr);
 		break;
+	case SYBSINT1:
+		length = tds_convert_int1((const int8_t *) src, desttype, cr);
+		break;
 	case SYBINT1:
 	case SYBUINT1:
-		length = tds_convert_int1((const TDS_TINYINT *) src, desttype, cr);
+		length = tds_convert_uint1((const TDS_TINYINT *) src, desttype, cr);
 		break;
 	case SYBINT2:
 		length = tds_convert_int2((const TDS_SMALLINT *) src, desttype, cr);
@@ -2261,12 +2326,12 @@ string_to_datetime(const char *instr, TDS_UINT len, int desttype, CONV_RESULT *
 		return sizeof(TDS_TIME);
 	}
 	if (desttype == SYB5BIGTIME) {
-		cr->bigtime = dt_time * (TDS_UINT8) 1000000u + t.tm_ns / 1000u;
+		cr->bigtime = dt_time * UINT64_C(1000000) + t.tm_ns / 1000u;
 		return sizeof(TDS_BIGTIME);
 	}
 	if (desttype == SYB5BIGDATETIME) {
-		cr->bigdatetime = (dt_days + BIGDATETIME_BIAS) * ((TDS_UINT8) 86400u * 1000000u)
-				  + dt_time * (TDS_UINT8) 1000000u + t.tm_ns / 1000u;
+		cr->bigdatetime = (dt_days + BIGDATETIME_BIAS) * (UINT64_C(86400) * 1000000u)
+				  + dt_time * UINT64_C(1000000) + t.tm_ns / 1000u;
 		return sizeof(TDS_BIGDATETIME);
 	}
 
@@ -2276,7 +2341,7 @@ string_to_datetime(const char *instr, TDS_UINT len, int desttype, CONV_RESULT *
 	cr->dta.date = dt_days;
 	cr->dta.has_time = 1;
 	cr->dta.time_prec = 7; /* TODO correct value */
-	cr->dta.time = ((TDS_UINT8) dt_time) * 10000000u + t.tm_ns / 100u;
+	cr->dta.time = dt_time * UINT64_C(10000000) + t.tm_ns / 100u;
 	return sizeof(TDS_DATETIMEALL);
 
 string_garbled:
@@ -2948,6 +3013,7 @@ tds_get_null_type(TDS_SERVER_TYPE srctype)
 		return SYBVARCHAR;
 		break;
 	case SYBINT1:
+	case SYBSINT1:
 	case SYBUINT1:
 	case SYBINT2:
 	case SYBINT4:
@@ -3367,7 +3433,7 @@ parse_int8(const char *buf, const char *pend, TDS_UINT8 * res, bool * p_negative
 	for (; digits; --digits, ++buf) {
 		/* add a digit to number and check for overflow */
 		TDS_UINT8 prev = num;
-		if (num > (((TDS_UINT8) 1u) << 63) / 5u)
+		if (num > (UINT64_C(1) << 63) / 5u)
 			return TDS_CONVERT_OVERFLOW;
 		num = num * 10u + (*buf - '0');
 		if (num < prev)
@@ -3396,11 +3462,11 @@ string_to_int8(const char *buf, const char *pend, TDS_INT8 * res)
 
 	/* check for overflow and convert unsigned to signed */
 	if (negative) {
-		if (num > (((TDS_UINT8) 1) << 63))
+		if (num > UINT64_C(1) << 63)
 			return TDS_CONVERT_OVERFLOW;
 		*res = 0 - num;
 	} else {
-		if (num >= (((TDS_UINT8) 1) << 63))
+		if (num >= UINT64_C(1) << 63)
 			return TDS_CONVERT_OVERFLOW;
 		*res = num;
 	}
